local _G = getfenv(0)
local L = _G.YurrCombatLogOptionsLocale
local core = YurrCombatLog
local db = core.db[core.db:GetCurrentProfile()]
local tab = LibStub("tekKonfig-TopTab")
local tekgroup = LibStub("tekKonfig-Group")
local tekcheck = LibStub("tekKonfig-Checkbox")
local tekbutt = LibStub("tekKonfig-Button")
local filter

local COMBATLOG_EVENT_LIST = {
	"ENVIRONMENTAL_DAMAGE",
	"SWING_DAMAGE",
	"SWING_MISSED",
	"RANGE_DAMAGE",
	"RANGE_MISSED",
	"SPELL_CAST_START",
	"SPELL_CAST_SUCCESS",
	"SPELL_CAST_FAILED",
	"SPELL_MISSED",
	"SPELL_DAMAGE",
	"SPELL_HEAL",
	"SPELL_ENERGIZE",
	"SPELL_DRAIN",
	"SPELL_LEECH",
	"SPELL_SUMMON",
	"SPELL_RESURRECT",
	"SPELL_CREATE",
	"SPELL_INSTAKILL",
	"SPELL_INTERRUPT",
	"SPELL_EXTRA_ATTACKS",
	"SPELL_DURABILITY_DAMAGE",
	"SPELL_DURABILITY_DAMAGE_ALL",
	"SPELL_AURA_APPLIED",
	"SPELL_AURA_APPLIED_DOSE",
	"SPELL_AURA_REMOVED",
	"SPELL_AURA_REMOVED_DOSE",
	"SPELL_AURA_BROKEN",
	"SPELL_AURA_BROKEN_SPELL",
	"SPELL_AURA_REFRESH",
	"SPELL_DISPEL",
	"SPELL_STOLEN",
	"ENCHANT_APPLIED",
	"ENCHANT_REMOVED",
	"SPELL_PERIODIC_MISSED",
	"SPELL_PERIODIC_DAMAGE",
	"SPELL_PERIODIC_HEAL",
	"SPELL_PERIODIC_ENERGIZE",
	"SPELL_PERIODIC_DRAIN",
	"SPELL_PERIODIC_LEECH",
	"SPELL_DISPEL_FAILED",
	"DAMAGE_SHIELD",
	"DAMAGE_SHIELD_MISSED",
	"DAMAGE_SPLIT",
	"PARTY_KILL",
	"UNIT_DIED",
	"UNIT_DESTROYED",
	"SPELL_BUILDING_DAMAGE",
	"SPELL_BUILDING_HEAL",
};
local UNITFLAG_LIST = {
	--Controller affiliation
	"COMBATLOG_OBJECT_AFFILIATION_MINE",
	"COMBATLOG_OBJECT_AFFILIATION_PARTY",
	"COMBATLOG_OBJECT_AFFILIATION_RAID",
	"COMBATLOG_OBJECT_AFFILIATION_OUTSIDER",
	--Reaction
	"COMBATLOG_OBJECT_REACTION_FRIENDLY",
	"COMBATLOG_OBJECT_REACTION_NEUTRAL",
	"COMBATLOG_OBJECT_REACTION_HOSTILE",
	--Controller
	"COMBATLOG_OBJECT_CONTROL_PLAYER",
	"COMBATLOG_OBJECT_CONTROL_NPC",
	--Type
	"COMBATLOG_OBJECT_TYPE_PLAYER",
	"COMBATLOG_OBJECT_TYPE_NPC",
	"COMBATLOG_OBJECT_TYPE_PET",
	"COMBATLOG_OBJECT_TYPE_GUARDIAN",
	"COMBATLOG_OBJECT_TYPE_OBJECT",
	--Special cases (non-exclusive)
	"COMBATLOG_OBJECT_TARGET",
	"COMBATLOG_OBJECT_FOCUS",
	"COMBATLOG_OBJECT_MAINTANK",
	"COMBATLOG_OBJECT_MAINASSIST",
	"COMBATLOG_OBJECT_RAIDTARGET1",
	"COMBATLOG_OBJECT_RAIDTARGET2",
	"COMBATLOG_OBJECT_RAIDTARGET3",
	"COMBATLOG_OBJECT_RAIDTARGET4",
	"COMBATLOG_OBJECT_RAIDTARGET5",
	"COMBATLOG_OBJECT_RAIDTARGET6",
	"COMBATLOG_OBJECT_RAIDTARGET7",
	"COMBATLOG_OBJECT_RAIDTARGET8",
	"COMBATLOG_OBJECT_NONE",
}
	
local function setFilter()
	CombatLogResetFilter()
	
	for k,v in ipairs(db.filters) do
		local events = ""
		local sUnit = 0
		local dUnit = 0
		if v.active then
			for k2,v2 in pairs(v) do
				if k2 ~= "name" and k2 ~= "active" and k2 ~= "sourceUnit" and k2 ~= "destUnit" then
					events = events .. ", " .. k2
				end
			end
			for k2,v2 in pairs(v.sourceUnit) do
				if v2 then
					sUnit = bit.bor(sUnit, _G[k2])
				end
			end
			for k2,v2 in pairs(v.destUnit) do
				if v2 then
					dUnit = bit.bor(dUnit, _G[k2])
				end
			end
			if events == "" then sUnit = nil end
			if sUnit == 0 then sUnit = nil end
			if dUnit == 0 then dUnit = nil end
			CombatLogAddFilter(events, sUnit, dUnit)
		end
		
		ChatFrame1:AddMessage("Active:" .. tostring(v.active) .. tostring(events) .. ", " .. tostring(sUnit) .. ", " .. tostring(dUnit))
	end
	
end
local function createFilter(name)
	local t = {
		name = name,
		active = false,
		ENVIRONMENTAL_DAMAGE = true,
		SWING_DAMAGE = true,
		SWING_MISSED = true,
		RANGE_DAMAGE = true,
		RANGE_MISSED = true,
		SPELL_CAST_START = false,
		SPELL_CAST_SUCCESS = false,
		SPELL_CAST_FAILED = false,
		SPELL_MISSED = true,
		SPELL_DAMAGE = true,
		SPELL_HEAL = true,
		SPELL_ENERGIZE = true,
		SPELL_DRAIN = true,
		SPELL_LEECH = true,
		SPELL_SUMMON = true,
		SPELL_RESURRECT = true,
		SPELL_CREATE = true,
		SPELL_INSTAKILL = true,
		SPELL_INTERRUPT = true,
		SPELL_EXTRA_ATTACKS = true,
		SPELL_DURABILITY_DAMAGE = false,
		SPELL_DURABILITY_DAMAGE_ALL = false,
		SPELL_AURA_APPLIED = true,
		SPELL_AURA_APPLIED_DOSE = true,
		SPELL_AURA_REMOVED = true,
		SPELL_AURA_REMOVED_DOSE = true,
		SPELL_AURA_BROKEN = true,
		SPELL_AURA_BROKEN_SPELL = true,
		SPELL_AURA_REFRESH = true,
		SPELL_DISPEL = true,
		SPELL_STOLEN = true,
		ENCHANT_APPLIED = true,
		ENCHANT_REMOVED = true,
		SPELL_PERIODIC_MISSED = true,
		SPELL_PERIODIC_DAMAGE = true,
		SPELL_PERIODIC_HEAL = true,
		SPELL_PERIODIC_ENERGIZE = true,
		SPELL_PERIODIC_DRAIN = true,
		SPELL_PERIODIC_LEECH = true,
		SPELL_DISPEL_FAILED = true,
		DAMAGE_SHIELD = true,
		DAMAGE_SHIELD_MISSED = true,
		DAMAGE_SPLIT = true,
		PARTY_KILL = true,
		UNIT_DIED = true,
		UNIT_DESTROYED = true,
		SPELL_BUILDING_DAMAGE = true,
		SPELL_BUILDING_HEAL = true,
		sourceUnit = {
			--Controller affiliation
			COMBATLOG_OBJECT_AFFILIATION_MINE = false,
			COMBATLOG_OBJECT_AFFILIATION_PARTY = false,
			COMBATLOG_OBJECT_AFFILIATION_RAID = false,
			COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = false,
			--Reaction
			COMBATLOG_OBJECT_REACTION_FRIENDLY = false,
			COMBATLOG_OBJECT_REACTION_NEUTRAL = false,
			COMBATLOG_OBJECT_REACTION_HOSTILE = false,
			--Controller
			COMBATLOG_OBJECT_CONTROL_PLAYER = false,
			COMBATLOG_OBJECT_CONTROL_NPC = false,
			--Type
			COMBATLOG_OBJECT_TYPE_PLAYER = false,
			COMBATLOG_OBJECT_TYPE_NPC = false,
			COMBATLOG_OBJECT_TYPE_PET = false,
			COMBATLOG_OBJECT_TYPE_GUARDIAN = false,
			COMBATLOG_OBJECT_TYPE_OBJECT = false,
			--Special cases (non-exclusive)
			COMBATLOG_OBJECT_TARGET = false,
			COMBATLOG_OBJECT_FOCUS = false,
			COMBATLOG_OBJECT_MAINTANK = false,
			COMBATLOG_OBJECT_MAINASSIST = false,
			COMBATLOG_OBJECT_RAIDTARGET1 = false,
			COMBATLOG_OBJECT_RAIDTARGET2 = false,
			COMBATLOG_OBJECT_RAIDTARGET3 = false,
			COMBATLOG_OBJECT_RAIDTARGET4 = false,
			COMBATLOG_OBJECT_RAIDTARGET5 = false,
			COMBATLOG_OBJECT_RAIDTARGET6 = false,
			COMBATLOG_OBJECT_RAIDTARGET7 = false,
			COMBATLOG_OBJECT_RAIDTARGET8 = false,
			COMBATLOG_OBJECT_NONE = false,
		},
		destUnit = {
			--Controller affiliation
			COMBATLOG_OBJECT_AFFILIATION_MINE = false,
			COMBATLOG_OBJECT_AFFILIATION_PARTY = false,
			COMBATLOG_OBJECT_AFFILIATION_RAID = false,
			COMBATLOG_OBJECT_AFFILIATION_OUTSIDER = false,
			--Reaction
			COMBATLOG_OBJECT_REACTION_FRIENDLY = false,
			COMBATLOG_OBJECT_REACTION_NEUTRAL = false,
			COMBATLOG_OBJECT_REACTION_HOSTILE = false,
			--Controller
			COMBATLOG_OBJECT_CONTROL_PLAYER = false,
			COMBATLOG_OBJECT_CONTROL_NPC = false,
			--Type
			COMBATLOG_OBJECT_TYPE_PLAYER = false,
			COMBATLOG_OBJECT_TYPE_NPC = false,
			COMBATLOG_OBJECT_TYPE_PET = false,
			COMBATLOG_OBJECT_TYPE_GUARDIAN = false,
			COMBATLOG_OBJECT_TYPE_OBJECT = false,
			--Special cases (non-exclusive)
			COMBATLOG_OBJECT_TARGET = false,
			COMBATLOG_OBJECT_FOCUS = false,
			COMBATLOG_OBJECT_MAINTANK = false,
			COMBATLOG_OBJECT_MAINASSIST = false,
			COMBATLOG_OBJECT_RAIDTARGET1 = false,
			COMBATLOG_OBJECT_RAIDTARGET2 = false,
			COMBATLOG_OBJECT_RAIDTARGET3 = false,
			COMBATLOG_OBJECT_RAIDTARGET4 = false,
			COMBATLOG_OBJECT_RAIDTARGET5 = false,
			COMBATLOG_OBJECT_RAIDTARGET6 = false,
			COMBATLOG_OBJECT_RAIDTARGET7 = false,
			COMBATLOG_OBJECT_RAIDTARGET8 = false,
			COMBATLOG_OBJECT_NONE = false,
		},
	},
	tinsert(db.filters, t)
end

local frame = CreateFrame("Frame", nil, InterfaceOptionsFramePanelContainer)
frame.name = L.filters
frame.parent = "YurrCombatLog"
frame:Hide()
frame:SetScript("OnShow", function(frame)
	local filter = 1
	
	local title = LibStub("tekKonfig-Heading").new(frame, L.filters)
	
	local group = tekgroup.new(frame, nil, "TOP", title, "BOTTOM", 0, -80)
	group:SetPoint("LEFT", 18, 0)
	group:SetPoint("BOTTOMRIGHT", -18, 18)
	
	---------------------
	------- Events ------
	---------------------
	local eventScrollFrame = CreateFrame("ScrollFrame", "eventScrollFrame", group, "UIPanelScrollFrameTemplate")
	local eventScrollChild = CreateFrame("Frame", "eventScrollChild", eventScrollFrame)
	eventScrollFrame:SetScrollChild(eventScrollChild)
	eventScrollFrame:SetPoint("TOPLEFT", group, "TOPLEFT", 8, -8)
	eventScrollFrame:SetPoint("BOTTOMRIGHT", -32, 8)
	eventScrollChild:SetWidth(300)  
	eventScrollChild:SetHeight(557)
	eventScrollFrame:EnableMouse(true)
	
	local eRow = 0
	local eSwitch = false
	local eventChecks = {}
	for k,v in ipairs(COMBATLOG_EVENT_LIST) do
		if not eSwitch then
			eventChecks[v] = tekcheck.new(eventScrollChild, nil, L[v], "TOPLEFT", eventScrollChild, "TOPLEFT", 0, -eRow*26)
			local checksound = eventChecks[v]:GetScript("OnClick")
			eventChecks[v]:SetScript("OnClick", function(self) checksound(self); db.filters[filter][v] = not db.filters[filter][v] end)
			eventChecks[v]:SetChecked(db.filters[filter][v])
		else
			eventChecks[v] = tekcheck.new(eventScrollChild, nil, L[v], "TOPLEFT", eventScrollChild, "TOPLEFT", 175, -eRow*26)
			local checksound = eventChecks[v]:GetScript("OnClick")
			eventChecks[v]:SetScript("OnClick", function(self) checksound(self); db.filters[filter][v] = not db.filters[filter][v] end)
			eventChecks[v]:SetChecked(db.filters[filter][v])
			eRow = eRow + 1
		end
		eSwitch = not eSwitch
	end
	
	---------------------
	----- unit ----
	---------------------
	local unitScrollFrame = CreateFrame("ScrollFrame", "unitScrollFrame", group, "UIPanelScrollFrameTemplate")
	local unitScrollChild = CreateFrame("Frame", "unitScrollChild", unitScrollFrame)
	unitScrollFrame:SetScrollChild(unitScrollChild)
	unitScrollFrame:SetPoint("TOPLEFT", group, "TOPLEFT", 8, -8)
	unitScrollFrame:SetPoint("BOTTOMRIGHT", -32, 8)
	unitScrollChild:SetWidth(300)  
	unitScrollChild:SetHeight(625)
	unitScrollFrame:EnableMouse(true)
	unitScrollFrame:Hide()
	
	local sRow = 0
	local sSwitch = false
	local flagChecks = {}
	for k,v in ipairs(UNITFLAG_LIST) do
		if not sSwitch then
			flagChecks[v] = tekcheck.new(unitScrollChild, nil, L[v], "TOPLEFT", unitScrollChild, "TOPLEFT", 0, -sRow*26)
			local checksound = flagChecks[v]:GetScript("OnClick")
			flagChecks[v]:SetScript("OnClick", function(self) checksound(self); db.filters[filter].sourceUnit[v] = not db.filters[filter].sourceUnit[v] end)
			flagChecks[v]:SetChecked(db.filters[filter].sourceUnit[v])
		else
			flagChecks[v] = tekcheck.new(unitScrollChild, nil, L[v], "TOPLEFT", unitScrollChild, "TOPLEFT", 175, -sRow*26)
			local checksound = flagChecks[v]:GetScript("OnClick")
			flagChecks[v]:SetScript("OnClick", function(self) checksound(self); db.filters[filter].sourceUnit[v] = not db.filters[filter].sourceUnit[v] end)
			flagChecks[v]:SetChecked(db.filters[filter].sourceUnit[v])
			sRow = sRow + 1
		end
		sSwitch = not sSwitch
	end
	
	local tab1 = tab.new(frame, L.events, "TOPLEFT", title, "BOTTOMLEFT", 0, -60)
	local tab2 = tab.new(frame, L.source, "LEFT", tab1, "RIGHT", -15, 0)
	local tab3 = tab.new(frame, L.dest, "LEFT", tab2, "RIGHT", -15, 0)
	local tab = 1
	tab2:Deactivate()
	tab3:Deactivate()
	tab1:SetScript("OnClick", function(self)
		tab = 1
		self:Activate()
		tab2:Deactivate()
		tab3:Deactivate()
		tradeview = false
		eventScrollFrame:Show()
		unitScrollFrame:Hide()
	end)
	tab2:SetScript("OnClick", function(self)
		tab = 2
		self:Activate()
		tab1:Deactivate()
		tab3:Deactivate()
		tradeview = true
		eventScrollFrame:Hide()
		unitScrollFrame:Show()
		for k,v in ipairs(UNITFLAG_LIST) do
			flagChecks[v]:SetChecked(db.filters[filter].sourceUnit[v])
		end
	end)
	tab3:SetScript("OnClick", function(self)
		tab = 3
		self:Activate()
		tab1:Deactivate()
		tab2:Deactivate()
		tradeview = true
		eventScrollFrame:Hide()
		unitScrollFrame:Show()
		for k,v in ipairs(UNITFLAG_LIST) do
			flagChecks[v]:SetChecked(db.filters[filter].destUnit[v])
		end
	end)
	
	local active = tekcheck.new(frame, nil, L.active, "TOPLEFT", title, "BOTTOMLEFT", 343, -25)
	local checksound = active:GetScript("OnClick")
	active:SetScript("OnClick", function(self) checksound(self); db.filters[filter].active = not db.filters[filter].active setFilter() end)
	active:SetChecked(db.filters[filter].active)
	
	-- Filter dropdown
	local FiltersDropdown, FiltersDropdowntext, FiltersDropdowncontainer = LibStub("tekKonfig-Dropdown").new(frame, nil, "TOPLEFT", title, "BOTTOMLEFT", -2, 0)
	FiltersDropdowntext:SetText(db.filters[filter].name)
	FiltersDropdown:SetWidth(170)
	local function FiltersOnClick(self)
		local f = string.gsub(self:GetName(), "DropDownList1Button(%d+)", "%1")
		filter = tonumber(f)
		if IsShiftKeyDown() then db.filters[filter] = nil end
		UIDropDownMenu_SetSelectedValue(FiltersDropdown, self.value)
		FiltersDropdowntext:SetText(self.value)
		if tab == 1 then
			for k,v in ipairs(COMBATLOG_EVENT_LIST) do
				eventChecks[v]:SetChecked(db.filters[filter][v])
			end
		elseif tab == 2 then
			for k,v in ipairs(UNITFLAG_LIST) do
				flagChecks[v]:SetChecked(db.filters[filter].sourceUnit[v])
			end
		elseif tab == 3 then
			for k,v in ipairs(UNITFLAG_LIST) do
				flagChecks[v]:SetChecked(db.filters[filter].destUnit[v])
			end
		end
		active:SetChecked(db.filters[filter].active)
	end
	local function initDropdown()
		local selected, info = UIDropDownMenu_GetSelectedValue(FiltersDropdown) or db.filters[filter].name, UIDropDownMenu_CreateInfo()

		for k, v in pairs(db.filters) do
			info.text = v.name
			info.value = v.name
			info.func = FiltersOnClick
			info.checked = (v.name) == selected
			UIDropDownMenu_AddButton(info)
		end
	end
	UIDropDownMenu_Initialize(FiltersDropdown, initDropdown)
	
	local refilter = tekbutt.new(frame, "TOPLEFT", FiltersDropdown, "TOPRIGHT", 0, -3)
	refilter:SetText(L.refilter)
	refilter.name = L.refilter
	refilter:SetScript("OnClick", function() Blizzard_CombatLog_Refilter() end)
	
	
	local prompt = CreateFrame('Frame', nil, UIParent)
	prompt:SetFrameStrata('DIALOG')
	prompt:SetToplevel(true)
	prompt:EnableMouse(true)
	prompt:SetClampedToScreen(true)
	prompt:SetMovable(false)
	prompt:SetResizable(false)
	prompt:SetWidth(360)
	prompt:SetHeight(80)
	prompt:SetBackdrop{
		bgFile='Interface\\DialogFrame\\UI-DialogBox-Background' ,
		edgeFile='Interface\\DialogFrame\\UI-DialogBox-Border',
		tile = true,
		insets = {left = 11, right = 12, top = 12, bottom = 11},
		tileSize = 32,
		edgeSize = 32,
	}
	prompt:SetPoint('TOP', 0, -220)
	prompt:Hide()
	local tr = prompt:CreateTitleRegion()
	tr:SetAllPoints(prompt)
	local header = prompt:CreateTexture(nil, 'ARTWORK')
	header:SetTexture('Interface\\DialogFrame\\UI-DialogBox-Header')
	header:SetWidth(326); header:SetHeight(64)
	header:SetPoint('TOP', 0, 12)
	local title = prompt:CreateFontString('ARTWORK')
	title:SetFontObject('GameFontNormal')
	title:SetPoint('TOP', header, 'TOP', 0, -14)
	title:SetText(L.addFilter)
	
	local function OnEnterPressed(self)
		local name = self:GetText()
		if name == "" then return end
		local filter = {}
		filter.name = name
		filter.active = false
		for k,v in ipairs(COMBATLOG_EVENT_LIST) do
			filter[v] = false
		end
		filter.sourceUnit = {}
		for k,v in ipairs(UNITFLAG_LIST) do
			filter.sourceUnit[v] = false
		end
		filter.destUnit = {}
		for k,v in ipairs(UNITFLAG_LIST) do
			filter.destUnit[v] = false
		end
		tinsert(db.filters, filter)
		initDropdown()
		self:ClearFocus(); prompt:Hide()
	end
	
	local editbox = CreateFrame("EditBox", nil, prompt, "InputBoxTemplate")
	editbox:SetAutoFocus(false)
	editbox:SetFontObject(ChatFontNormal)
	editbox:SetScript("OnEscapePressed", function() this:ClearFocus(); prompt:Hide() end)
	editbox:SetScript("OnEnterPressed", OnEnterPressed)
	editbox:SetTextInsets(0,0,3,3)
	editbox:SetMaxLetters(256)
	editbox:SetPoint("BOTTOMLEFT", prompt, "BOTTOMLEFT", 21, 25)
	editbox:SetPoint("RIGHT", prompt, "RIGHT", -106, 0)
	editbox:SetHeight(19)

	local addEvent = CreateFrame('CheckButton', nil, editbox, 'OptionsButtonTemplate')
	addEvent:SetText(L.addFilter)
	addEvent:SetScript('OnClick', function() OnEnterPressed(editbox) end)
	addEvent:SetPoint('TOPLEFT', editbox, "TOPRIGHT", 0, 2)
	
	local addFilter = tekbutt.new(frame, "TOPLEFT", refilter, "TOPRIGHT", 0, 0)
	addFilter:SetText(L.addFilter)
	addFilter.name = L.addFilter
	addFilter:SetScript("OnClick", function() prompt:Show(); editbox:SetText(""); editbox:SetFocus() end)
	
	frame:SetScript("OnShow", nil)
end)
InterfaceOptions_AddCategory(frame)
